@@ -1,19 +1,24 @@
 
 from __future__ import unicode_literals
-import os, sys, re
+
+import os
+import sys
+import re
 import codecs
 import inspect
 import uuid
 import datetime
+import json
+from json import JSONEncoder
+from collections import OrderedDict
+from collections import namedtuple
+from pyld import jsonld
+from rdflib import ConjunctiveGraph
 
 KEY_ORDER_DEFAULT = 10000
 LINKED_ART_CONTEXT_URI = "https://linked.art/ns/v1/linked-art.json"
 
 # 2.5 and 2.6 are very out of date. Assume 2.7 or better
-import json
-from collections import OrderedDict, namedtuple
-
-from json import JSONEncoder
 
 try:
 	STR_TYPES = [str, unicode] #Py2.7
@@ -24,29 +29,26 @@ except:
 	FILE_STREAM_CLASS = io.TextIOBase
 
 
-from pyld import jsonld
 pyld_proc = jsonld.JsonLdProcessor()
 min_context = {
-    "crm": "http://www.cidoc-crm.org/cidoc-crm/", 
-    "sci": "http://www.ics.forth.gr/isl/CRMsci/", 
-    "rdf": "http://www.w3.org/1999/02/22-rdf-syntax-ns#", 
-    "rdfs": "http://www.w3.org/2000/01/rdf-schema#", 
-    "dc": "http://purl.org/dc/elements/1.1/", 
-    "dcterms": "http://purl.org/dc/terms/", 
-    "schema": "http://schema.org/", 
-    "skos": "http://www.w3.org/2004/02/skos/core#", 
-    "foaf": "http://xmlns.com/foaf/0.1/", 
-    "xsd": "http://www.w3.org/2001/XMLSchema#", 
-    "dig": "http://www.ics.forth.gr/isl/CRMdig/", 
-    "la": "https://linked.art/ns/terms/", 
-    "id": "@id", 
+	"crm": "http://www.cidoc-crm.org/cidoc-crm/",
+    "sci": "http://www.ics.forth.gr/isl/CRMsci/",
+    "rdf": "http://www.w3.org/1999/02/22-rdf-syntax-ns#",
+    "rdfs": "http://www.w3.org/2000/01/rdf-schema#",
+    "dc": "http://purl.org/dc/elements/1.1/",
+    "dcterms": "http://purl.org/dc/terms/",
+    "schema": "http://schema.org/",
+    "skos": "http://www.w3.org/2004/02/skos/core#",
+    "foaf": "http://xmlns.com/foaf/0.1/",
+    "xsd": "http://www.w3.org/2001/XMLSchema#",
+    "dig": "http://www.ics.forth.gr/isl/CRMdig/",
+    "la": "https://linked.art/ns/terms/",
+    "id": "@id",
     "type": "@type"
 }
 re_bnodes = re.compile("^_:b([0-9]+) ", re.M)
 re_bnodeo = re.compile("> _:b([0-9]+) <", re.M)
 re_quad = re.compile(" <[^<]+?> .$", re.M)
-from rdflib import ConjunctiveGraph
-
 
 PropInfo = namedtuple("PropInfo", [
 	'property', # the name of the property, eg 'identified_by'
@@ -89,7 +91,6 @@ class ProfileError(MetadataError):
 	"""Raised when a class or property not in the configured profile is used"""
 	pass
 
-
 class CromJsonEncoder(JSONEncoder):
 
 	def default(self, o):
@@ -99,7 +100,6 @@ class CromJsonEncoder(JSONEncoder):
 		else:
 			return JSONEncoder.default(self, o)			
 
-
 class CromulentFactory(object):
 
 	def __init__(self, base_url="", base_dir="", lang="", full_names=False, 
@@ -311,7 +311,12 @@ class CromulentFactory(object):
 		else:
 			raise ConfigurationError("Unknown auto-id type")
 
-		return self.base_url + what.__class__._uri_segment + "/" + str(slug)		
+		if what.__class__._uri_segment:
+			seg = what.__class__._uri_segment + "/"
+		else:
+			seg = ""
+
+		return self.base_url + seg + str(slug)	
 
 	def find_serializable(self, what):
 
@@ -498,7 +503,10 @@ class CromulentFactory(object):
 					g.bind(k, v)
 			g.parse(data=data, format="nquads")
 			out = g.serialize(format=format)
-			return out.decode('utf-8')
+			if type(out) == bytes:
+				return out.decode('utf-8')
+			else:
+				return out
 
 	def get_filename(self, whatid, extension=""):
 
@@ -649,6 +657,8 @@ class ExternalResource(object):
 				if len(curied) == 2 and curied[0] in self._factory.prefixes:
 					self.id = ident
 					self._full_id = self._factory.prefixes[curied[0]] + curied[1]	
+				elif not self.__class__._uri_segment:
+					self.id = factory.base_url + ident
 				else:
 					self.id = factory.base_url + self.__class__._uri_segment + "/" + ident
 		elif factory.auto_assign_id:
@@ -771,7 +781,7 @@ class BaseResource(ExternalResource):
 		 
 	def _check_prop(self, which, value):
 		val_props = self._factory.validate_properties
-		val_profile = self._factory.validate_profile
+		val_profile = self._factory.validate_profile and getattr(self, '_validate_profile', True)
 		val_range = self._factory.validate_range
 		for c in self._classhier:
 			if which in c._all_properties:
