@@ -6,12 +6,13 @@ import json
 
 class Reader(object):
 
-	def __init__(self):
+	def __init__(self, validate_props=True, validate_profile=True):
 		self.uri_object_map = {}
 		self.forward_refs = []
 		self.vocab_props = ['assigned_property']
 		self.vocab_classes = {}
-		self.validate_props = True
+		self.validate_profile = validate_profile
+		self.validate_props = validate_props
 
 		for cx in dir(vocab):
 			what = getattr(vocab, cx)
@@ -52,7 +53,7 @@ class Reader(object):
 			if uri in self.uri_object_map:
 				setattr(what, prop, self.uri_object_map[uri])
 			else:
-				raise NotImplementedError("No class information for %s.%s = %s".format(what, prop, uri))
+				raise NotImplementedError("No class information for %s.%s = %s" % (what, prop, uri))
 
 	def construct(self, js):
 		# pass in json, get back object
@@ -73,8 +74,8 @@ class Reader(object):
 				raise DataError("Resource %s has unknown class %s" % (ident, typ) )
 
 		# now check vocab.ext_classes to try and refine
+		trash = None 
 		if 'classified_as' in js:
-			trash = None 
 			for c in js['classified_as']:
 				i = c.get('id', '')
 				clx2 = self.vocab_classes.get((typ, i), None)
@@ -82,10 +83,9 @@ class Reader(object):
 					clx = clx2
 					trash = c
 					break
-			if trash is not None:
-				js['classified_as'].remove(trash)
 
 		what = clx(ident=ident)
+		what._validate_profile = self.validate_profile
 		self.uri_object_map[ident] = what
 
 		if self.validate_props:
@@ -98,18 +98,26 @@ class Reader(object):
 		for (prop, value) in itms:
 			if prop in ['id', 'type']:
 				continue
+
 			if self.validate_props and not prop in propList:
 				raise DataError("Unknown property %s on %s" % (prop, clx.__name__))
 
 			# Climb looking for range
+			rng = None
 			for c in what._classhier:		
 				if prop in c._all_properties:
 					rng = c._all_properties[prop].range
 					break
 
+			if not rng:
+				print(f"Couldn't find range for {prop} on {what}; trashing it")
+				continue
+
 			if type(value) != list:
 				value = [value]
 			for subvalue in value:
+				if trash is not None and prop == 'classified_as' and subvalue == trash:
+					continue
 				if rng == str:
 					setattr(what, prop, subvalue)				
 				elif type(subvalue) == dict or isinstance(subvalue, OrderedDict):
