@@ -79,7 +79,7 @@ ext_classes = {
 	"StockNumber": {"parent": Identifier, "id": "300412177", "label": "Stock Number"},
 	
 	"EmailAddress": {"parent": Identifier, "id":"300435686", "label": "Email Address"},
-	"StreetAddress": {"parent": Identifier, "id":"300435687", "label": "Street Address"},
+	"StreetAddress": {"parent": Identifier, "id":"300386983", "label": "Street Address"},
 	"TelephoneNumber": {"parent": Identifier, "id": "300435688", "label": "Telephone Number"},
 	"FaxNumber": {"parent": Identifier, "id": "300435689", "label": "Fax Number"},
 	"StreetNumber": {"parent": Identifier, "id":"300419272", "label": "Street Number"},
@@ -135,6 +135,7 @@ ext_classes = {
 	"PaginationStatement": {"parent": LinguisticObject, "id":"300435440", "label": "Pagination Statement", "metatype": "brief text"},
 	"FoliationStatement": {"parent": LinguisticObject, "id":"300435441", "label": "Foliation Statement", "metatype": "brief text"},
 	"LanguageStatement": {"parent": LinguisticObject, "id": "300435433", "label": "Language Statement", "metatype": "brief text"},
+	"ResearchStatement": {"parent": LinguisticObject, "id": "300265639", "label": "Research Statement", "metatype": "brief text"},
 
 	"TranscribedInscription": {"parent": LinguisticObject, "id": "300028702", "label": "Transcribed Inscription", "metatype": "brief text"},
 	"TranscribedSignature": {"parent": LinguisticObject, "id": "300028705", "label": "Transcribed Signature", "metatype": "brief text"},
@@ -187,10 +188,19 @@ ext_classes = {
 	"ExhibitionPlace": {"parent": Place, "id":"300005748", "label": "Exhibition (place)"},
 	"CityBlock":    {"parent": Place, "id":"300008077", "label": "City Block"},
 	"City":         {"parent": Place, "id":"300008389", "label": "City"},
+	"Township": 	{"parent": Place, "id":"300000792", "label": "Township"},
+	"County":		{"parent": Place, "id":"300000771", "label": "County"},
+	"State": 		{"parent": Place, "id":"300000776", "label": "State"},
 	"Province":     {"parent": Place, "id":"300000774", "label": "Province"},
+	"Region": 		{"parent": Place, "id":"300236112", "label": "Region"},
 	"Nation":       {"parent": Place, "id":"300128207", "label": "Nation"},
 	"Sovereign":	{"parent": Place, "id":"300232420", "label": "Sovereign State"},
 	"StoragePlace": {"parent": Place, "id":"300150151", "label": "Storage (place)"},
+	"Continent":    {"parent": Place, "id": "300128176","label": "Continent"},
+	"Country":      {"parent": Place, "id":"300128207", "label": "Country"},	
+	"Ocean": {"parent": Place, "id":"300008687", "label": "Ocean"},
+	"Sea": {"parent": Place, "id":"300008694", "label": "Sea"},
+	"Bay": {"parent": Place, "id":"300132316", "label": "Bay"},
 
 	"Building":     {"parent": HumanMadeObject, "id": "300004792", "label":"Building"},
 
@@ -205,6 +215,8 @@ ext_classes = {
 	"Atelier":     {"parent": Group, "id":"300404277", "label": "Atelier"},
 	"FollowerGroup": {"parent": Group, "id":"300404282", "label": "Follower"},
 	"PupilGroup":  {"parent": Group, "id":"300404279", "label": "Pupil"},
+	"Family":      {"parent": Group, "id": "300055474", "label":"Family"},
+	"Organization":{"parent": Group, 'id':'300025948', 'label':"Organization"}, 
 
 	"Nationality": {"parent": Type, "id":"300379842", "label": "Nationality"},
 	"Gender":      {"parent": Type, "id":"300055147", "label": "Gender"},
@@ -274,9 +286,10 @@ ext_classes = {
 	"ArchiveGroupSet": {"parent": Set, "id":"300404022", "label": "Archival Grouping"},
 	"ArchiveSubGroupSet": {"parent": Set, "id":"300404023", "label": "Archival SubGrouping"},
 	
-	"PrimaryName": {"parent": Name, "id":"300404670", "label": "Primary Name"},
-	"SortName": {"parent": Name, "id":"300404672", "label": "Sorting Name"},
+	"PrimaryName": {"parent": Name, "id":"300404670", "label": "Primary Name"}, # preferred 
+	"AlternateName": {"parent": Name, "id":"300264273", "label":"Alternate Name"}, # not preferred
 
+	"SortName": {"parent": Name, "id":"300404672", "label": "Sorting Name"},
 	"Title": {"parent": Name, "id":"300195168", "label": "Title"},
 	"Subtitle": {"parent": Name, "id":"300312006", "label":"Subtitle"},
 	"DisplayName": {"parent": Name, "id": "300404669", "label": "Display Title"},
@@ -284,6 +297,7 @@ ext_classes = {
 
 	"PersonalName": {"parent": Name, "id":"300266386", "label": "Personal Name"},
 	"AliasName": {"parent": Name, "id": "300404664", "label": "Alias"},
+	"Pseudonym": {"parent": Name, "id": "300404657", "label": "Psueudonym"},
 	"MaidenName": {"parent": Name, "id": "300404682", "label": "Maiden Name"},
 	"FormerName": {"parent": Name, "id": "300435719", "label": "Former Name"},
 	"GivenName": {"parent": Name, "id":"300404651", "label": "Given Name"},
@@ -427,56 +441,64 @@ identity_instances = {
 	"partsUnit": {"parent": MeasurementUnit, "id": "300404159", "label": "parts"},
 
 	# Languages
-	"english": {"parent": Language, "id": "300388277", "label": "English"},
-	"french": {"parent": Language, "id":"300388306","label": "French"},
-	"german": {"parent": Language, "id":"300388344","label": "German"},
-	"spanish": {"parent": Language, "id":"300389311","label": "Spanish"},
-	"chinese": {"parent": Language, "id":"300388113", "label":"Chinese"},
-	"hindi": {"parent": Language, "id":"300388412", "label":"Hindi"},
-	"afrikaans": {"parent": Language, "id":"300387782", "label":"Afrikaans"},
-	"albanian": {"parent": Language, "id":"300387803", "label":"Albanian"},
-	"arabic": {"parent": Language, "id":"300387843", "label":"Arabic"},
-	"bulgarian": {"parent": Language, "id":"300388034", "label":"Bulgarian"},
-	"bosnian": {"parent": Language, "id":"300388023", "label":"Bosnian"},
-	"catalan": {"parent": Language, "id":"300388072", "label":"Catalan"},
-	"portuguese":{"parent": Language, "id":"300389115", "label":"Portuguese"},
-	"bengali": {"parent": Language, "id":"300387971", "label":"Bengali"},
-	"russian": {"parent": Language, "id":"300389168", "label":"Russian"},
-	"dutch": {"parent": Language, "id":"300388256", "label":"Dutch"},
-	"finnish": {"parent": Language, "id":"300388299", "label":"Finnish"},
-	"icelandic": {"parent": Language, "id":"300388449", "label":"Icelandic"},
-	"irish": {"parent": Language, "id":"300388467", "label":"Irish"},
-	"italian": {"parent": Language, "id":"300388474", "label":"Italian"},
-	"farsi": {"parent": Language, "id":"300388296", "label":"Farsi"},
-	"greek": {"parent": Language, "id":"300389734", "label":"Greek"},
-	"gujarati": {"parent": Language, "id":"300388371", "label":"Gujarati"},
-	"ancient greek": {"parent": Language, "id": "300387827", "label": "Ancient Greek"},
-	"korean": {"parent": Language, "id":"300388633", "label":"Korean"},
-	"latin": {"parent": Language, "id":"300388693", "label":"Latin"},
-	"lithuanian": {"parent": Language, "id":"300388723", "label":"Lithuanian"},
-	"macedonian": {"parent": Language, "id":"300388760", "label":"Macedonian"},
-	"japanese": {"parent": Language, "id":"300388486", "label":"Japanese"},
-	"croatian": {"parent": Language, "id": "300388185", "label": "Croatian"},
-	"czech": {"parent": Language, "id": "300388191", "label": "Czech"},
-	"danish": {"parent": Language, "id": "300388204", "label": "Danish"},
-	"greek": {"parent": Language, "id": "300388361", "label": "Greek"},
-	"georgian": {"parent": Language, "id": "300388343", "label": "Georgian"},
-	"hebrew": {"parent": Language, "id": "300388401", "label": "Hebrew"},
-	"hungarian": {"parent": Language, "id": "300388770", "label": "Magyar (Hungarian)"},
-	"norwegian": {"parent": Language, "id": "300388992", "label": "Norwegian"},
-	"polish": {"parent": Language, "id": "300389109", "label": "Polish"},
-	"romanian": {"parent": Language, "id": "300389157", "label": "Romanian"},
-	"serbo-croatian": {"parent": Language, "id": "300389248", "label": "Serbo-Croatian"},
-	"slovak": {"parent": Language, "id": "300389290", "label": "Slovak"},
-	"slovenian": {"parent": Language, "id": "300389291", "label": "Slovenian"},
-	"serbian": {"parent": Language, "id": "300389247", "label": "Serbian"},
-	"swedish": {"parent": Language, "id": "300389336", "label": "Swedish"},
-	"turkish": {"parent": Language, "id": "300389470", "label": "Turkish"},
-	"welsh": {"parent": Language, "id": "300389555", "label": "Welsh"},
+	"english": {"parent": Language, "id": "300388277", "label": "English", 'code':'en'},
+	"french": {"parent": Language, "id":"300388306","label": "French", 'code':'fr'},
+	"german": {"parent": Language, "id":"300388344","label": "German", 'code':'de'},
+	"spanish": {"parent": Language, "id":"300389311","label": "Spanish", 'code':'es'},
+	"chinese": {"parent": Language, "id":"300388113", "label":"Chinese", 'code':'zh'},
+	"hindi": {"parent": Language, "id":"300388412", "label":"Hindi", 'code':'hi'},
+	"afrikaans": {"parent": Language, "id":"300387782", "label":"Afrikaans", 'code':'af'},
+	"albanian": {"parent": Language, "id":"300387803", "label":"Albanian", 'code':'sq'},
+	"arabic": {"parent": Language, "id":"300387843", "label":"Arabic", 'code':'ar'},
+	"bulgarian": {"parent": Language, "id":"300388034", "label":"Bulgarian", 'code':'bg'},
+	"bosnian": {"parent": Language, "id":"300388023", "label":"Bosnian", 'code':'bs'},
+	"catalan": {"parent": Language, "id":"300388072", "label":"Catalan", 'code':'ca'},
+	"portuguese":{"parent": Language, "id":"300389115", "label":"Portuguese", 'code':'pt'},
+	"bengali": {"parent": Language, "id":"300387971", "label":"Bengali", 'code':'bn'},
+	"russian": {"parent": Language, "id":"300389168", "label":"Russian", 'code':'ru'},
+	"dutch": {"parent": Language, "id":"300388256", "label":"Dutch", 'code':'nl'},
+	"finnish": {"parent": Language, "id":"300388299", "label":"Finnish", 'code':'fi'},
+	"icelandic": {"parent": Language, "id":"300388449", "label":"Icelandic", 'code':'is'},
+	"irish": {"parent": Language, "id":"300388467", "label":"Irish", 'code':'ga'},
+	"italian": {"parent": Language, "id":"300388474", "label":"Italian", 'code':'it'},
+	"farsi": {"parent": Language, "id":"300388296", "label":"Farsi", 'code':'fa'},
+	"gujarati": {"parent": Language, "id":"300388371", "label":"Gujarati", 'code':'gu'},
+	"ancient greek": {"parent": Language, "id": "300387827", "label": "Ancient Greek", 'code':'grc'},
+	"korean": {"parent": Language, "id":"300388633", "label":"Korean", 'code':'ko'},
+	"latin": {"parent": Language, "id":"300388693", "label":"Latin", 'code':'la'},
+	"lithuanian": {"parent": Language, "id":"300388723", "label":"Lithuanian", 'code':'lt'},
+	"macedonian": {"parent": Language, "id":"300388760", "label":"Macedonian", 'code':'mk'},
+	"japanese": {"parent": Language, "id":"300388486", "label":"Japanese", 'code':'ja'},
+	"croatian": {"parent": Language, "id": "300388185", "label": "Croatian", 'code':'hr'},
+	"czech": {"parent": Language, "id": "300388191", "label": "Czech", 'code':'cs'},
+	"danish": {"parent": Language, "id": "300388204", "label": "Danish", 'code':'da'},
+	"greek": {"parent": Language, "id": "300388361", "label": "Greek", 'code':'el'},
+	"georgian": {"parent": Language, "id": "300388343", "label": "Georgian", 'code':'ka'},
+	"hebrew": {"parent": Language, "id": "300388401", "label": "Hebrew", 'code':'he'},
+	"hungarian": {"parent": Language, "id": "300388770", "label": "Magyar (Hungarian)", 'code':'hu'},
+	"norwegian": {"parent": Language, "id": "300388992", "label": "Norwegian", 'code':'no'},
+	"polish": {"parent": Language, "id": "300389109", "label": "Polish", 'code':'pl'},
+	"romanian": {"parent": Language, "id": "300389157", "label": "Romanian", 'code':'ro'},
+	"serbo-croatian": {"parent": Language, "id": "300389248", "label": "Serbo-Croatian", 'code':''},
+	"slovak": {"parent": Language, "id": "300389290", "label": "Slovak", 'code':'sk'},
+	"slovenian": {"parent": Language, "id": "300389291", "label": "Slovenian", 'code':'sl'},
+	"serbian": {"parent": Language, "id": "300389247", "label": "Serbian", 'code':'sr'},
+	"swedish": {"parent": Language, "id": "300389336", "label": "Swedish", 'code':'sv'},
+	"turkish": {"parent": Language, "id": "300389470", "label": "Turkish", 'code':'tr'},
+	"welsh": {"parent": Language, "id": "300389555", "label": "Welsh", 'code':'cy'},
+	"luxembourgish": {"parent": Language, "id": "300443689", "label": "Luxembourgish", "code": "lb"},
+	"urdu": {"parent": Language, "id": "300389502", "label": "Urdu", 'code':'ur'},
+	"swahili": {"parent": Language, "id": "300443741", "label": "Swahili", 'code':'sw'},
+	"indonesian": {"parent": Language, "id": "300388460", "label": "Indonesian", "code": "id"},
+	"telugu": {"parent": Language, "id": "300389389", "label": "Telugu", "code": "te"},
+	"tamil": {"parent": Language, "id": "300389365", "label": "Tamil", "code": "ta"},
+	"thai": {"parent": Language, "id": "300389405", "label": "Thai", "code": "th"},
+	"marathi": {"parent": Language, "id": "300388821", "label": "Marathi", "code": "mr"},
+	"punjabi": {"parent": Language, "id": "300389057", "label": "Punjabi", "code": "pa"},
 
 	# Currencies
-	"us dollars": {"parent": Currency, "id":"300411994", "label": "US Dollars"},
-	"gb pounds": {"parent": Currency, "id":"300411998", "label": "British Pounds"},
+	"us dollars": {"parent": Currency, "id":"300411994", "label": "US Dollars", 'code':'USD'},
+	"gb pounds": {"parent": Currency, "id":"300411998", "label": "British Pounds", 'code':'GBP'},
 	"gb shillings": {"parent": Currency, "id":"300412167", "label": "British Shillings"},
 	"dutch guilder": {"parent": Currency, "id":"300412019", "label": "Dutch Guilder"},
 	"fr assignats": {"parent": Currency, "id":"300412157", "label": "French Assignats"},
@@ -532,6 +554,7 @@ identity_instances = {
 	"sovereign": {"parent": Type, "id": "300232420", "label": "Sovereign State"},
 	"building": {"parent": Type, "id": "300004792", "label": "Building"},
 	"city block": {"parent": Type, "id": "300008077", "label": "City Block"},
+	"address": {'parent': Type, 'id': '300386983', 'label': 'Street Address'},
 
 	# dot one qualifiers
 	"style of": {"parent": Type, "id": "300404285", "label": "Style Of"},
@@ -745,6 +768,12 @@ def add_attribute_assignment_check():
 		if assto_res:
 			p177_res = getattr(self, p177, None)
 			assto_res._check_prop(p177_res, value)
+
+		current = getattr(self, ass, None)
+		if current:
+			value = [*current, value]
+		elif type(value) is not list:
+			value = [value]
 		object.__setattr__(self, ass, value)
 	setattr(AttributeAssignment, "set_%s" % ass, aa_set_assigned)
 
@@ -753,7 +782,8 @@ def add_attribute_assignment_check():
 		p177_res = getattr(self, p177, None)		
 		if ass_res and p177_res:
 			# unmap the URI to property name
-			value._check_prop(p177_res, ass_res)
+			for ar in ass_res:
+				value._check_prop(p177_res, ar)
 		object.__setattr__(self, assto, value)
 	setattr(AttributeAssignment, "set_%s" % assto, aa_set_assigned_to)
 
@@ -761,7 +791,8 @@ def add_attribute_assignment_check():
 		ass_res = getattr(self, ass, None)
 		assto_res = getattr(self, assto, None)
 		if ass_res and assto_res:
-			assto_res._check_prop(value, ass_res)
+			for ar in ass_res:
+				assto_res._check_prop(value, ar)
 		object.__setattr__(self, p177, value)
 	setattr(AttributeAssignment, "set_%s" % p177, aa_set_assigned_property_type)
 
@@ -803,11 +834,23 @@ def add_linked_art_boundary_check():
 		elif isinstance(value, ProvenanceEntry):
 			return False
 
-		if rel in ["part", "member"]:
+		boundary_crossing_props = set([
+			"part_of", 'member_of', "specific_purpose", "caused_by",
+			"starts_before_the_end_of",
+			"ends_after_the_start_of",
+			"starts_before_the_start_of",
+			"starts_after_the_start_of",
+			"ends_before_the_start_of",
+			"starts_after_the_end_of",
+			"ends_before_the_end_of",
+			"ends_after_the_end_of",
+		])
+		
+		if rel in ["part", "member", "specific_purpose_of", "caused"]:
 			# Downwards, internal simple partitioning 
 			# This catches an internal part to a LinguisticObject
 			return True
-		elif rel in ["part_of", 'member_of']:
+		elif rel in boundary_crossing_props:
 			# upwards partition refs are inclusion, and always boundary crossing
 			return False
 		elif value.type in boundary_classes:
